import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;
import java.lang.*;

public class WinEncoder extends JPanel {

    public static void main(String[] args) {
	new WinEncoder();
    }

    final JFrame frame = new JFrame("Problem Set Encoder for Windows");
    final String description = "Problem Set Encoder v0.2 for Windows by tlin@mit.edu";
    final String instruction_message = "\"Problem Set Encoder\" is for creating XML problem sets to\nbe used by the WOPSM application.\n\nThis Windows problem set Encoder requires that you import\nPostscript files for each title/problem statement/question/\nanswer/hint. Before using this Encoder, you should have the\nPostscript files ready. You can generate such files from Microsoft\nWord. For instance, create a new document, type the word\n\"Problem Set 1\" into it, then print it to a Postscript File. Then,\nrename it so it has a .ps extension.\n\nTo start a new XML problem set, click File>New. You can\nalso File>Save problem sets, and File>Load them later.\n\nIf an item is in red, that means it has not been set yet.\nBy clicking on the item, you are brought to a screen where\nyou can set the value.\n\nIf an item is in blue, that means its value has been set. You\ncan still click on a blue item and edit it. To remove an item\nin blue, edit it and set its value to \"delete\". That will \nmake the item red (unset) again.\n\nIf an item is in black, that means it has no value to set. You\ncan delete an item in black by clicking on it, then choosing\nto delete it.\n\nWhen you are done with your problem set, move it into the\nWOPSM directory and run the Problem Set Converter program.";
    final String faq_message = "Question: How does WOPSM relate to LOPSM?\nAnswer: LOPSM (Latex Online Problem Set Maker) is for the\nAthena/Linux environment, and has the user to input LaTeX\ntext instead of loading Postscript files. WOPSM can display\nLOPSM-generated files, and LOPSM can display WOPSM-\ngenerated files.";

    //JScrollPane overall = new JScrollPane();

    public WinEncoder() {
	frame.setSize(500,600);
	
	Container content = frame.getContentPane();

	tabs.add("Overall", getOverallView());
	content.add(tabs);

	frame.setJMenuBar(getMenuBar());
	frame.addWindowListener(closeOnX());

	frame.setVisible(true);
    }

    JTabbedPane tabs = new JTabbedPane();

    public JPanel getOverallView() {
	JPanel overallPanel = new JPanel();

	// the overall view starts out blank

	return overallPanel;
    }

    public JMenuBar getMenuBar() {
	JMenuBar mb = new JMenuBar();

	JMenu file = new JMenu("File");
	mb.add(file);

	JMenuItem newpset = new JMenuItem("New");
	newpset.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    currentpset = new ProblemSet();
		    //tabs.setSelectedIndex(0);       no tabs in windows version
		    repaintOverall();
		}
	    });
	file.add(newpset);

	JMenuItem loadpset = new JMenuItem("Open");
	loadpset.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    JFileChooser jfc = new JFileChooser(".");
		    int result = jfc.showOpenDialog(frame);
		    if (result == jfc.APPROVE_OPTION) {
			loadpsetfile(jfc.getSelectedFile());
			//tabs.setSelectedIndex(0);    no tabs in windows version
		    }
		}
	    });
	file.add(loadpset);

	JMenuItem savepset = new JMenuItem("Save");
	savepset.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    if (currentpset == null) {
			popup("Nothing to save!");
		    } else {
			JFileChooser jfc = new JFileChooser(".");

			int proceed = 0;
			while (proceed == 0) {
			    int result = jfc.showSaveDialog(frame);

			    if (result != jfc.APPROVE_OPTION) {
				return;   // user cancelled from "select file" box
			    } else {
				proceed = savepsetfile(jfc.getSelectedFile());

				if (proceed == 1) {
				    //tabs.setSelectedIndex(0);  no tabs in windows version
				    return;    // data was saved
				} else if (proceed == 2) {
				    return;    // user cancelled from "overwrite file?" box
				}
				// proceed can also be 0, which means the user wants to choose again
			    }
			}
		    }
		}
	    });
	file.add(savepset);
	
	file.addSeparator();

	JMenuItem exit = new JMenuItem("Exit");
	exit.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    int choice = JOptionPane.showConfirmDialog(frame, "Are you sure you want to Exit?");
		    if (choice == 0) {
			System.exit(0);
		    }
		}
	    });
	file.add(exit);

	JMenu help = new JMenu("Help");
	mb.add(help);

	JMenuItem instructions = new JMenuItem("Instructions");
	instructions.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    // create a non-modal JDialog, so they can keep the instructions
		    // open while using the program
		    
                    final JDialog dialog = new JDialog(frame, "Instructions  (this window can be kept open)");

		    JTextArea label = new JTextArea(instruction_message);
		    label.setEditable(false);

                    JButton closeButton = new JButton("OK");
                    closeButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            dialog.setVisible(false);
                            dialog.dispose();
                        }
                    });
		    JPanel closePanel = new JPanel();
		    closePanel.add(closeButton);

                    JPanel contentPane = new JPanel(new BorderLayout());
                    contentPane.add(label, BorderLayout.CENTER);
                    contentPane.add(closePanel, BorderLayout.SOUTH);
                    contentPane.setOpaque(true);
                    dialog.setContentPane(contentPane);

                    //Show it.
                    dialog.setSize(new Dimension(400, 540));
                    dialog.setLocationRelativeTo(frame);
                    dialog.setVisible(true);
		}
	    });
	help.add(instructions);

	JMenuItem faq = new JMenuItem("Frequently Asked Questions");
	faq.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    // create a non-modal JDialog, so they can keep the instructions
		    // open while using the program
		    
                    final JDialog dialog = new JDialog(frame, "FAQ  (this window can be kept open)");

		    JTextArea label = new JTextArea(faq_message);
		    label.setEditable(false);

                    JButton closeButton = new JButton("OK");
                    closeButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            dialog.setVisible(false);
                            dialog.dispose();
                        }
                    });
		    JPanel closePanel = new JPanel();
		    closePanel.add(closeButton);

                    JPanel contentPane = new JPanel(new BorderLayout());
                    contentPane.add(label, BorderLayout.CENTER);
                    contentPane.add(closePanel, BorderLayout.SOUTH);
                    contentPane.setOpaque(true);
                    dialog.setContentPane(contentPane);

                    //Show it.
                    dialog.setSize(new Dimension(400, 200));
                    dialog.setLocationRelativeTo(frame);
                    dialog.setVisible(true);
		}
	    });
	help.add(faq);

	JMenuItem about = new JMenuItem("About");
	about.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    popup(description);
		}
	    });
	help.add(about);

	return mb;
    }

    public void popup(String message) {
	JOptionPane.showMessageDialog(frame, message);
    }

    ProblemSet currentpset;

    public void loadpsetfile(File file) {
	// read the problem set into a string
	try {	    
	    StringBuffer resultString = new StringBuffer();
	    
	    FileReader fr = new FileReader(file);
	    BufferedReader br = new BufferedReader(fr);
	    String line;

	    while ((line = br.readLine())!=null) {
		resultString.append(line + "\n");
		line = "";
	    }
	    br.close();
		   
	    currentpset = new ProblemSet(file, resultString.toString());
	    repaintOverall();

	} catch (Exception e) {
	    popup(e.toString());
	}
    }

    public int savepsetfile(File file) {
	// flatten the problem set, and save it to a file
	try {
	    String flattened = currentpset.getFlatString();

	    if (file.exists()) {
		int choice = JOptionPane.showConfirmDialog(frame, "Overwrite existing file?");
		// choice == 0 means that the user wants to overwrite, so proceed
		if (choice == 1) { return 0; }  // let the user choose another file
		if (choice == 2) { return 2; }  // user cancelled
	    }
	    FileWriter fw = new FileWriter(file);
	    fw.write(flattened);
	    fw.close();
	    return 1;    // file saved sucessfully

	} catch (Exception e) {
	    popup(e.toString());
	    return 2;    // treat it as if user cancelled
	}
    }

    public void repaintOverall() {
	JPanel updatedJPanel = currentpset.getJPanel();
	JScrollPane sp = new JScrollPane(updatedJPanel);
	tabs.setComponentAt(0, sp);    
    } 

    public WindowAdapter closeOnX() {
	return new WindowAdapter() {
		public void windowClosing(WindowEvent we) {
		    System.exit(0);
		}
	    };};

    public boolean notempty(String totest) {
	totest = totest.trim();
	if (totest.length() == 0) { return false; };
	if (totest.equals("\n")) { return false; };
	return true;
    }

    class ProblemSet {
	File file;
	String filetext;

	XMLstring xmlversion;
	Title title;
	Vector problems; // of Problems

	public ProblemSet(File file, String filetext) {
	    this.file = file;
	    this.filetext = filetext;
	    this.xmlversion = extractXMLversion();
	    this.title = extractTitle();
	    this.problems = extractProblems();
	}

	public ProblemSet() {
	    this.file = null;
	    this.filetext = null;
	    this.xmlversion = new XMLstring();
	    this.title = new Title();
	    this.problems = new Vector();
	}

	// create the string version for saving to a file
	public String getFlatString() {
	    String ret = new String();
	    
	    ret += xmlversion.toFileString()+"\n";  // we need an input box string
	    ret += "<ProblemSet>\n";
	    ret += title.toFileString()+"\n";       // we need a title too
	    for (int i=0; i<problems.size(); i++) {
		ret += ((Problem)problems.elementAt(i)).getFlatString()+"\n";
	    }
	    ret += "</ProblemSet>\n";

	    return ret;
	}

	private XMLstring extractXMLversion() {
	    return new XMLstring(extractLine(filetext,"?xml"));
	}

	private Title extractTitle() {
	    return new Title(extractBlock(filetext,"Title","/Title"));
	}

	private Vector extractProblems() {
	    Vector problemStrings = extractMultiBlock(filetext,"Problem","/Problem");
	    Vector ret = new Vector();
	    for (int i=0; i<problemStrings.size(); i++) {
		ret.add(new Problem( (String)problemStrings.elementAt(i) ));
	    }
	    return ret;
	}

	public JPanel getJPanel() {
	    JPanel ret = new JPanel();

	    GridBagLayout gridbag = new GridBagLayout();
	    GridBagConstraints c = new GridBagConstraints();
	    ret.setLayout(gridbag);

	    overallgridy = -1;
	    //addEditableButton(ret, gridbag, c, "xml header", 0, xmlversion);   (they don't get to edit the XML header in the windows version)
	    addDeletableButton(ret, gridbag, c, "problem set", 0, null, -1);
	    addEditableButton(ret, gridbag, c, "title", 15, title);

	    for (int i=0; i<problems.size(); i++) {
		addDeletableButton(ret, gridbag, c, "problem "+(i+1), 15, problems, i);

		/*
		JLabel newlabel = new JLabel("problem "+(i+1));
		overallgridy++; c.gridx = 0; c.gridy = overallgridy;
		c.anchor = GridBagConstraints.WEST;
		c.insets = new Insets(0,15,0,0);
		gridbag.setConstraints(newlabel, c);
		ret.add(newlabel);
		*/

		((Problem)problems.elementAt(i)).addToJPanel(ret, gridbag, c);
	    }

	    // let the user add a new problem
	    //
	    JButton newbutton = new JButton("add new problem");
	    newbutton.setForeground(colorOfAddButton);
	    newbutton.setBorder(BorderFactory.createEmptyBorder());   
	    overallgridy++; c.gridx = 0; c.gridy = overallgridy;
	    c.anchor = GridBagConstraints.WEST;
	    c.insets = new Insets(0,15,0,0);
	    gridbag.setConstraints(newbutton, c);
	    ret.add(newbutton);
	    newbutton.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent ae) {
			// Create the new button
			Problem newproblem = new Problem();
			problems.add(newproblem);
			repaintOverall();
		    }
		});

	    return ret;
	}
    }

    public int overallgridy = 0;

    public void addDeletableButton(JPanel ret, GridBagLayout gridbag, GridBagConstraints c, String buttonname, int insetvalue, Vector toDeleteFrom, int indexToDelete) {
	JButton newbutton = new JButton(buttonname);
	newbutton.setBorder(BorderFactory.createEmptyBorder());   
	overallgridy++; c.gridx = 0; c.gridy = overallgridy;
	c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(0,insetvalue,0,0);
	gridbag.setConstraints(newbutton, c);
	ret.add(newbutton);
	final Vector toDeleteFromCopy = toDeleteFrom;
	final int indexToDeleteCopy = indexToDelete;
	newbutton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    if (toDeleteFromCopy == null) {
			popup("This item cannot be deleted.");
		    } else {
			int choice = JOptionPane.showConfirmDialog(frame, "Would you like to delete this item?");
			if (choice == 0) {    // yes
			    toDeleteFromCopy.removeElementAt(indexToDeleteCopy);
			    repaintOverall();
			}
		    }
		}
	    });
    }

    public void addEditableButton(JPanel ret, GridBagLayout gridbag, GridBagConstraints c, String buttonname, int insetvalue, Writable toWrite) {
	JButton newbutton = new JButton(buttonname);
	if (toWrite.toString() == null) {
	    newbutton.setForeground(Color.red);
	} else {
	    newbutton.setForeground(Color.blue);
	};
	newbutton.setBorder(BorderFactory.createEmptyBorder());   
	overallgridy++; c.gridx = 0; c.gridy = overallgridy;
	c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(0,insetvalue,0,0);
	gridbag.setConstraints(newbutton, c);
	ret.add(newbutton);
	final Writable toWriteCopy = toWrite;
	newbutton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    // this is where it goes if the user edits an Editable button

		    String original_entry = toWriteCopy.toString();

		    JFileChooser jfc = new JFileChooser(".");
		    jfc.setDialogTitle("Choose a Postscript File");
		    if (original_entry != null) {
			jfc.setSelectedFile(new File(original_entry));
		    }

		    int result = jfc.showOpenDialog(frame);

		    if (result == jfc.APPROVE_OPTION) {
			String delete_check = jfc.getSelectedFile().getName();
			if (delete_check.equals("delete")) {
			    toWriteCopy.setValue(null);
			} else if (!delete_check.substring(delete_check.length()-3).equals(".ps")) {
			    popup("Sorry, change not made.\nYour filename must end\nin a postscript (\".ps\")\nextension.");
			} else {
			    String chosen_file = jfc.getSelectedFile().getAbsolutePath();
			    toWriteCopy.setValue(chosen_file);
			}
			repaintOverall();
		    }
		}
	    });
    }
    
    class Writable {
	/*	protected String value = null;
	protected String newvalue = "uninitialized new value error";
	
	public void setValue(String s) {
	    value = s;
	}

	public String toString() {
	    return value;
	}

	public String toInputBoxString() {
	    if (value != null) {
		return value;
	    } else {
		return newvalue;
	    }
	    }*/

	protected String wrapper_front = null;
	protected String wrapper_end = null;
	protected String value = null;

	public void setValue(String s) {
	    value = s;
	}

	public String toString() {
	    return value;
	}

	public String toFileString() {
	    if (value == null) {
		return wrapper_front + "unspecified" + wrapper_end;
	    } else {
		return wrapper_front + value + wrapper_end;
	    }
	}

	// to file string unless null
	public String toFileStringUN() {
	    if (value == null) {
		return null;
	    } else {
		return wrapper_front + value + wrapper_end;
	    }
	}
    }

    class XMLstring extends Writable {
	public XMLstring(String xml) {
	    this.wrapper_front = "";
	    this.wrapper_end = "";
	    this.value = xml;
	}

	public XMLstring() {
	    this.wrapper_front = "";
	    this.wrapper_end = "";
	    this.value = "<?xml version=\"1.0\" standalone='yes'?>";
	}
    }

    class Title extends Writable {
	public Title(String title) {
	    this.wrapper_front = "<Title>\n<![CDATA[\n";
	    this.wrapper_end = "\n]]>\n</Title>";
	    this.value = striptags(title);
	}

	public Title() {
	    this.wrapper_front = "<Title>\n<![CDATA[\n";
	    this.wrapper_end = "\n]]>\n</Title>";
	    this.value = null;
	}
    }

    class Problem {
	String text;
	Statement statement;
	Vector questionGroups; // of QuestionGroups

	public Problem(String text) {
	    this.text = text;
	    this.statement = extractStatement();
	    this.questionGroups = extractQuestionGroups();
	}

	public Problem() {
	    this.text = null;
	    this.statement = new Statement();
	    this.questionGroups = new Vector();
	}

	public String getFlatString() {
	    String ret = new String();
	    ret += "<Problem>\n";

	    ret += statement.toFileString()+"\n";  // we need a problem statement if there's a problem there

	    /*
	    String statementString = statement.toString();
	    if (statementString == null) { statementString = ""; };
	    ret += statementString+"\n";   // we don't necessarily need any statements
	    */

	    for (int i=0; i<questionGroups.size(); i++) {
		ret += ((QuestionGroup)questionGroups.elementAt(i)).getFlatString()+"\n";
	    }
	    ret += "</Problem>";
	    return ret;
	}

	public Statement extractStatement() {
	    return new Statement(extractBlock(text,"Statement","/Statement"));
	}

	public Vector extractQuestionGroups() {
	    Vector questionStrings = extractMultiBlock(text,"QuestionGroup","/QuestionGroup");
	    Vector ret = new Vector();
	    for (int i=0; i<questionStrings.size(); i++) {
		ret.add(new QuestionGroup( (String)questionStrings.elementAt(i) ));
	    }
	    return ret;
	}

	public void addToJPanel(JPanel ret, GridBagLayout gridbag, GridBagConstraints c) {
	    addEditableButton(ret, gridbag, c, "problem statement", 30, statement);

	    // questiongroups labels
	    //
	    for (int i=0; i<questionGroups.size(); i++) {
		addDeletableButton(ret, gridbag, c, "question group "+(i+1), 30, questionGroups, i);
		/*
		JLabel newlabel = new JLabel("question group "+(i+1));
		overallgridy++; c.gridx = 0; c.gridy = overallgridy;
		c.anchor = GridBagConstraints.WEST;
		c.insets = new Insets(0,30,0,0);
		gridbag.setConstraints(newlabel, c);
		ret.add(newlabel);
		*/

		((QuestionGroup)questionGroups.elementAt(i)).addToJPanel(ret, gridbag, c);
	    }

	    // let the user add a new question group
	    //
	    JButton newbutton = new JButton("add new question group");
	    newbutton.setForeground(colorOfAddButton);
	    newbutton.setBorder(BorderFactory.createEmptyBorder());   
	    overallgridy++; c.gridx = 0; c.gridy = overallgridy;
	    c.anchor = GridBagConstraints.WEST;
	    c.insets = new Insets(0,30,0,0);
	    gridbag.setConstraints(newbutton, c);
	    ret.add(newbutton);
	    newbutton.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent ae) {
			// Create the new button
			QuestionGroup newqg = new QuestionGroup();
			questionGroups.add(newqg);
			repaintOverall();
		    }
		});
	}
    }

    class Statement extends Writable {
	public Statement(String statement) {
	    this.wrapper_front = "<Statement>\n<![CDATA[\n";
	    this.wrapper_end = "\n]]>\n</Statement>";
	    this.value = striptags(statement);
	}

	public Statement() {
	    this.wrapper_front = "<Statement>\n<![CDATA[\n";
	    this.wrapper_end = "\n]]>\n</Statement>";
	    this.value = null;
	}
    }

    class QuestionGroup {
	String text;
	Question question;
	Answer answer;
	Vector hints;

	public QuestionGroup(String text) {
	    this.text = text;
	    this.question = extractQuestion();
	    this.answer = extractAnswer();
	    this.hints = extractHints();
	}

	public QuestionGroup() {
	    this.text = null;
	    this.question = new Question();
	    this.answer = new Answer();
	    this.hints = new Vector();
	}

	public String getFlatString() {
	    String ret = new String();
	    ret += "<QuestionGroup>\n";
	    String questionString = question.toFileStringUN();
	    if (questionString == null) { questionString = ""; };
	    ret += questionString+"\n";   // we don't necessarily need a question

	    for (int i=0; i<hints.size(); i++) {
		// so empty hints don't get added
		String hintString = ((Hint)hints.elementAt(i)).toFileStringUN();
		if (hintString == null) { hintString = ""; };
		ret += hintString+"\n";
	    }

	    String answerString = answer.toFileStringUN();
	    if (answerString == null) { answerString = ""; };
	    ret += answerString+"\n";     // we don't necessarily need an answer

	    ret += "</QuestionGroup>";
	    return ret;
	}

	public Question extractQuestion() {
	    return new Question(extractBlock(text,"Question","/Question"));
	}

        public Answer extractAnswer() {
	    return new Answer(extractBlock(text,"Answer","/Answer"));
	}

	public Vector extractHints() {
	    Vector hintStrings = extractMultiBlock(text,"Hint","/Hint");
	    Vector ret = new Vector();
	    for (int i=0; i<hintStrings.size(); i++) {
		ret.add(new Hint( (String)hintStrings.elementAt(i) ));
	    }
	    return ret;
	}
	    
	public void addToJPanel(JPanel ret, GridBagLayout gridbag, GridBagConstraints c) {
	    addEditableButton(ret, gridbag, c, "question statement", 45, question);

	    // hint buttons
	    //
	    for (int i=0; i<hints.size(); i++) {
		JLabel newlabel = new JLabel("hint "+(i+1));

		addEditableButton(ret, gridbag, c, "hint "+(i+1), 45, (Hint)hints.elementAt(i));
	    }

	    // let the user add a new hint
	    //
	    JButton newbutton = new JButton("add new hint");
	    newbutton.setForeground(colorOfAddButton);
	    newbutton.setBorder(BorderFactory.createEmptyBorder());   
	    overallgridy++; c.gridx = 0; c.gridy = overallgridy;
	    c.anchor = GridBagConstraints.WEST;
	    c.insets = new Insets(0,45,0,0);
	    gridbag.setConstraints(newbutton, c);
	    ret.add(newbutton);
	    newbutton.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent ae) {
			// Create the new button
			Hint newhint = new Hint();
			hints.add(newhint);
			repaintOverall();
		    }
		});

	    addEditableButton(ret, gridbag, c, "answer statement", 45, answer);
	}
    }

    private final Color colorOfAddButton = new Color(255,160,160); // light red

    class Question extends Writable {
	public Question(String question) {
	    this.wrapper_front = "<Question>\n<![CDATA[\n";
	    this.wrapper_end = "\n]]>\n</Question>";
	    this.value = striptags(question);
	}

	public Question() {
	    this.wrapper_front = "<Question>\n<![CDATA[\n";
	    this.wrapper_end = "\n]]>\n</Question>";
	    this.value = null;
	}
    }
 
    class Answer extends Writable {
	public Answer(String answer) {
	    this.wrapper_front = "<Answer>\n<![CDATA[\n";
	    this.wrapper_end = "\n]]>\n</Answer>";
	    this.value = striptags(answer);
	}

	public Answer() {
	    this.wrapper_front = "<Answer>\n<![CDATA[\n";
	    this.wrapper_end = "\n]]>\n</Answer>";
	    this.value = null;
	}
    }

    class Hint extends Writable {
	public Hint(String hint) {
	    this.wrapper_front = "<Hint>\n<![CDATA[\n";
	    this.wrapper_end = "\n]]>\n</Hint>";
	    this.value = striptags(hint);
	}

	public Hint() {
	    this.wrapper_front = "<Hint>\n<![CDATA[\n";
	    this.wrapper_end = "\n]]>\n</Hint>";
	    this.value = null;
	}
    }

    protected static String striptags(String original) {
	int index1 = original.indexOf("[CDATA[\n");
	int index2 = original.indexOf("\n]]>");
	return original.substring(index1+8, index2);
    }

    // Extract a line of text beginning with "<starttext"
    //
    public String extractLine(String filetext, String starttext) {
	StringTokenizer st = new StringTokenizer(filetext, "<>");
	while (st.hasMoreTokens()) {
	    String token = st.nextToken();
	    if ((token.length()>=starttext.length()) && token.substring(0,starttext.length()).equals(starttext)) {
		return "<"+token+">";
	    }
	}
	popup("no xml version found");
	return null;
    }

    // Extract a block of text beginning with "<start>" and ending with "<end>"
    //
    public String extractBlock(String filetext, String start, String end) {
	StringTokenizer st = new StringTokenizer(filetext, "<>");
	while (st.hasMoreTokens()) {
	    String token = st.nextToken();
	    if (token.equals(start)) {
		String block = "<"+token+">"+"\n";
		while (st.hasMoreTokens()) {
		    String token2 = st.nextToken();
		    if (notempty(token2)) {
			block += "<"+token2+">"+"\n";
			if (token2.equals(end)) {
			    return block;
			}
		    }
		}
		popup("a "+start+" tag was unclosed");
		    return null;
	    }
	}
	//popup("no "+start+" tag found");
	return null;
    }

    // Extract a Vector of all blocks of text beginning with "<start>" and ending with "<end>"
    // returns an empty Vector if no such blocks were found
    //
    public Vector extractMultiBlock(String filetext, String start, String end) {
	Vector ret = new Vector();
	
	StringTokenizer st = new StringTokenizer(filetext, "<>");
	while (st.hasMoreTokens()) {
	    String token = st.nextToken();
	    checkpoint: if (token.equals(start)) {
		String block = "<"+token+">"+"\n";
		while (st.hasMoreTokens()) {
		    String token2 = st.nextToken();
		    if (notempty(token2)) {
			block += "<"+token2+">"+"\n";
			if (token2.equals(end)) {
			    ret.add(block);
			    break checkpoint;
			}
		    }
		}
		popup(start+" tag unclosed");
		return null;
	    }
	}
	return ret;
    }
}



/**
    JTextArea currenttext = new JTextArea();
    Writable currentmodify = new Writable();

    public JPanel getCurrentPanel() {
	// the panel for the "current"
	JPanel currentPanel = new JPanel();

	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	currentPanel.setLayout(gridbag);

	currenttext.setLineWrap (true);
	currenttext.setWrapStyleWord (true);

	JScrollPane sp = new JScrollPane(currenttext);
	c.gridx = 0; c.gridy = 0;
	c.ipadx = 400; c.ipady = 240;
	c.insets = new Insets(5,5,5,5);
	gridbag.setConstraints(sp, c);
	currentPanel.add(sp);

	JPanel okayAndCancel = new JPanel();
	okayAndCancel.setLayout(new GridLayout(0,2));
	JButton okay = new JButton("Okay");
	JButton cancel = new JButton("Cancel");
	okayAndCancel.add(okay);
	okayAndCancel.add(cancel);

	okay.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    if (currentmodify != null) {
			// they just clicked Okay, so we should update the value
			String newtext = currenttext.getText().trim();
			if (newtext.equals("")) {
			    // if it's blank, then maybe the user wanted to null it
			    currentmodify.setValue(null);		
			} else {
			    // otherwise, give them the untrimmed version of what they have
			    currentmodify.setValue(currenttext.getText());
			}
			repaintOverall();  // repaint in case it goes red->blue or blue->red
		    }
		    tabs.setSelectedIndex(0);
		}
	    });

	cancel.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    // they just clicked Cancel, so go back without changing anything
		    tabs.setSelectedIndex(0);
		}
	    });


	c.gridx = 0; c.gridy = 1;
	c.ipadx = 0; c.ipady = 0;
	c.insets = new Insets(0,0,0,0);
	c.anchor = GridBagConstraints.CENTER;
	gridbag.setConstraints(okayAndCancel, c);
	currentPanel.add(okayAndCancel);

	return currentPanel;
    }
**/
