import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.apache.xerces.jaxp.*;
import javax.xml.parsers.DocumentBuilder;
import java.io.*;
import javax.imageio.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

import vfl.PSImageData;

//import java.awt.*;
//import javax.swing.*;
import java.awt.event.*;

public class Converter {
    JTextArea instructions = new JTextArea();

    JLabel gs_label = new JLabel("");
    String gs_full = null;
    JButton gs_button = new JButton("Find gswin32c.exe      ");

    JLabel if_label = new JLabel("");
    String if_full = null;
    JButton if_button = new JButton("Specify Input XML File");

    boolean gs_specified = false;
    boolean xml_specified = false;

    Container content = new Container();

    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();


    public Converter() {
	final JFrame frame = new JFrame("Converter");
	frame.setSize(400,280);
	frame.addWindowListener(closeOnX());

	content = frame.getContentPane();

	content.setLayout(gridbag);

	instructions.setText("The XML problem sets need to be converted before they can\nbe read. In order to convert, I need to know the location of\nGhostscript. In particular, I need the directory that\ngswin32c.exe is in. This will usually be c:\\program files\\gs\\bin,\nbut may vary depending on where you installed it.");
	instructions.setEditable(false);

	c.anchor = GridBagConstraints.CENTER;
	c.gridwidth = 2;
	c.insets = new Insets(10,10,15,10);
	c.gridx = 0; c.gridy = 0;
	gridbag.setConstraints(instructions, c);
	content.add(instructions);

	c.anchor = GridBagConstraints.EAST;
	c.gridwidth = 1;
	c.insets = new Insets(5,5,5,5);
	c.gridx = 0; c.gridy = 1;
	gridbag.setConstraints(gs_button, c);
	content.add(gs_button);

	c.anchor = GridBagConstraints.WEST;
	c.gridx = 1; c.gridy = 1;
	c.insets = new Insets(0,10,0,0);
	gridbag.setConstraints(gs_label, c);
	content.add(gs_label);

	gs_button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    JFileChooser jfc = new JFileChooser(".");
		    if (gsbuf != null) { jfc.setSelectedFile(new File(gsbuf)); };
		    int result = jfc.showOpenDialog(frame);

		    if (result == jfc.APPROVE_OPTION) {
			gs_label.setText(jfc.getSelectedFile().getName());
			gs_full = jfc.getSelectedFile().getAbsolutePath();
			try {
			    FileWriter fw = new FileWriter("gs_location.txt");
			    fw.write(gs_full);
			    fw.close();
			} catch (Exception e) {}
			gs_specified = true;
			addbutton();
		    }
		}
	    });

	c.anchor = GridBagConstraints.EAST;
	c.gridx = 0; c.gridy = 2;
	c.insets = new Insets(5,5,5,5);
	gridbag.setConstraints(if_button, c);
	content.add(if_button);

	c.anchor = GridBagConstraints.WEST;
	c.gridx = 1; c.gridy = 2;
	c.insets = new Insets(0,10,0,0);
	gridbag.setConstraints(if_label, c);
	content.add(if_label);

	if_button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    JFileChooser jfc = new JFileChooser(".");
		    if (psbuf != null) { jfc.setSelectedFile(new File(psbuf)); };
		    int result = jfc.showOpenDialog(frame);

		    if (result == jfc.APPROVE_OPTION) {
			if_label.setText(jfc.getSelectedFile().getName());
			if_full = jfc.getSelectedFile().getAbsolutePath();
			try {
			    FileWriter fw = new FileWriter("ps_location.txt");
			    fw.write(if_full);
			    fw.close();
			} catch (Exception e) {}
			xml_specified = true;
			addbutton();
		    }
		}
	    });

	// check gs_location.txt and ps_location.txt for cached locations
	try {
	    FileReader fr = new FileReader("gs_location.txt");
	    BufferedReader br = new BufferedReader(fr);
	    gsbuf = br.readLine();
	    gsbuf = gsbuf.replaceAll("\n","");   
	    br.close();
	} catch (Exception e) {
	}

	try {
	    FileReader fr = new FileReader("ps_location.txt");
	    BufferedReader br = new BufferedReader(fr);
	    psbuf = br.readLine();
	    psbuf = psbuf.replaceAll("\n","");
	    br.readLine();
	    br.close();
	} catch (Exception e) {
	}

	frame.setVisible(true);
    }

    private static String gsbuf = null;
    private static String psbuf = null;

    private void addbutton() {
	if (gs_specified && xml_specified) {
	    JButton convert = new JButton("Convert!");

	    c.anchor = GridBagConstraints.CENTER;
	    c.gridwidth = 2;
	    c.insets = new Insets(15,10,10,10);
	    c.gridx = 0; c.gridy = 3;
	    gridbag.setConstraints(convert, c);
	    content.add(convert);

	    convert.addActionListener((new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    String inputXML = if_full;   // absolute path of input file
		    String tempDIR = "." + File.separator + "tmp";
	            String millis = "" + System.currentTimeMillis(); // milliseconds since 1970
		    String output = ".." + File.separator + "PSImages" + File.separator + if_label.getText() + "_" + millis;  // use input file name and millis since 1970
		    String gsDIR = gs_full;      // absolute path of ghostscript executable 

		    String[] args = { inputXML, tempDIR, output, gsDIR };
		    LatexProblemSetParser.main(args);
		}
	    }));
	}
    }

    public static void main(String[] args) {
	new Converter();
    }

    public WindowAdapter closeOnX() {
	return new WindowAdapter() {
		public void windowClosing(WindowEvent we) {
		    System.exit(0);
		}
	    };
    };
}

class LatexProblemSetParser {
   public static void main (String argv []) {
      try {
	  System.out.println("Starting conversion.");
	  System.out.println("arg 0: " + argv[0]);
	  System.out.println("arg 1: " + argv[1]);
	  System.out.println("arg 2: " + argv[2]);
	  System.out.println("arg 3: " + argv[3]);

	  //String[] args = { inputXML, tempDIR, output, gsDIR };

	  gsLocation = argv[3];

	  //System.out.println("Input Syntax: <input xmlfile> <tmpdir> <outputfile> -s (for Sun)");
	  //System.out.println("                                                    -l (for Linux)");
         File td =  new File ( (new File (argv[2])).getParent() + File.separator + "info.txt") ;
         if (td.isFile()) {
            BufferedReader istream = new BufferedReader(new FileReader(td));
            int a = Integer.parseInt(istream.readLine());
            Vector s = new Vector();
            for (int i = 0; i < a ; i++) {
		s.add(istream.readLine());
	    }
            int t = a+1;
            FileWriter ostream = new FileWriter(td);
            ostream.write((new Integer(t)).toString()+"\n");
            for (int i = 0; i < a ; i++) {
		ostream.write(((String)s.elementAt(i))+"\n");
	    }
            ostream.write( (new File (argv[2])).getName()+"\n");
            ostream.close();
	 }
	 else {
            FileWriter ostream = new FileWriter(td);
            ostream.write((new Integer(1)).toString()+"\n");
            ostream.write( (new File (argv[2])).getName()+"\n");
            ostream.close(); 
         }  
         DocumentBuilderFactoryImpl factory = new DocumentBuilderFactoryImpl();
         DocumentBuilder parser = factory.newDocumentBuilder();
         tmp = argv[1];
	 /**
         if (argv[3].compareTo("-l") == 0) 
	     sun = false;
         else
             sun = true;
	 **/

         Document doc = parser.parse(argv[0]);
         count(doc);  
         Vector images = new Vector();
         FileOutputStream ostream = new FileOutputStream(new File(argv[2]));
	 ObjectOutputStream p = new ObjectOutputStream(ostream);
	 p.writeInt(vs.size());
         for (int i = 0; i< vs.size();i++) {
           p.writeObject(vs.elementAt(i));
           p.flush();
         }
         p.close();
         ostream.flush();
	 ostream.close();

         FileInputStream istream = new FileInputStream(new File(argv[2]));
	 ObjectInputStream is = new ObjectInputStream(istream);
	 int sz = is.readInt();
         for (int i = 0; i < sz; i++) {
            PSImageData a = ((PSImageData)(is.readObject()));
            images.add(a);
         }
                
	 istream.close();
		   
         JFrame jf = new JFrame("OUTPUT");
         JScrollPane editorScrollPane = new JScrollPane( new GifPanel(images));
         editorScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
         jf.getContentPane().add(editorScrollPane);   
         jf.setSize(800,400);
         jf.setResizable(false);
         jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);       
         jf.show();
                  
      } catch (Exception e) {
	  e.printStackTrace();
      } 
   }
    private static String gsLocation;
    //private static boolean sun;
   private static String tmp;
   private static int qgnum=0;
   private static int probnum=0;
   private static int hintnum=1;
   private static int c = 0;
   private static boolean hintset = false;
   private static boolean titleset = false;
   private static int quest =0;
   private static Vector vs = new Vector();  
   public static void count(Node node) {
 
        // is there anything to do?
        if (node == null) {
            return;
        }

        int type = node.getNodeType();
        switch (type) {
            case Node.DOCUMENT_NODE: {           
                Document document = (Document)node;
                count(document.getDocumentElement());
                break;
            }

            case Node.ELEMENT_NODE: {
           
                //System.out.println ("\n\nTagN: " + ((Element)node).getTagName());
                if (((Element)node).getTagName().compareTo("QuestionGroup") == 0) {
                    qgnum++;
                    hintnum=1;
                    quest =0;
                    hintset = false; 
                    titleset = false;
                }
	        else if(((Element)node).getTagName().compareTo("Problem") == 0) {
                    probnum++;
                    qgnum = 0;
                    hintnum = 1;
                    quest = 0;               
                    hintset = false; 
                    titleset = false;
               } 
                else if(((Element)node).getTagName().compareTo("Statement") == 0) {
                    qgnum = 0;
                    hintnum = 1;
                    quest = 0;
                    hintset = false; 
                    titleset = false;                
                }

                else if(((Element)node).getTagName().compareTo("Title") == 0) {
                    titleset = true;                   
                }
	        else if(((Element)node).getTagName().compareTo("Hint") == 0) {
		    hintnum++;
                    hintset = true;
                   titleset = false;                
 
                } 
                else if(((Element)node).getTagName().compareTo("Question") == 0) {
		    quest=0;
                    hintset = false;
                   titleset = false;                
                 }
                else if(((Element)node).getTagName().compareTo("Answer") == 0) {
		    quest=1;
                    hintset = false;
                    titleset = false;                
                }
                Node child = node.getFirstChild();
                while (child != null) {
                    count(child);
                    child = child.getNextSibling();
                } 
                break;
	    }
            /*
            case Node.ENTITY_REFERENCE_NODE: {
                Node child = node.getFirstChild();
                while (child != null) {
                    count(child);
                    child = child.getNextSibling();
                }
                break;
            }
            */
	case Node.CDATA_SECTION_NODE: {
	       c++;            
                Text text = (Text)node;
                System.out.print(".");
                /*
                if (titleset)
                   System.out.println("ProbNUM: 0 \nQuestionGroup: 0\nQuest: 0\nContent:\n" + text.getData());
                else if (hintset)
                    System.out.println("ProbNUM: " + probnum + "\nQuestionGroup: " + qgnum + "\nQuest: " + hintnum + "\nContent:\n" + text.getData());
                else                   
		     System.out.println("ProbNUM: " + probnum + "\nQuestionGroup: " + qgnum + "\nQuest: " + quest + "\nContent:\n" + text.getData());
		*/
                try {
		    System.out.println("Processing Unit #" + c + "....");

		    String postScriptLocation = text.getData();
		    postScriptLocation = postScriptLocation.replaceAll("\n","");
		    if (postScriptLocation.equals("unspecified")) { postScriptLocation = "unspecified.ps"; };

		    /**
		    File lw = new File (tmp+File.separator+"latextmp.tex");
		    FileWriter sstream = new FileWriter(lw);
		    sstream.write(text.getData());		    sstream.flush();
		    sstream.close();
		    Runtime rt = Runtime.getRuntime();
		    String cmd = "latex " +tmp+File.separator+"latextmp.tex";

		    Process ct = rt.exec(cmd,null,new File(tmp));

		    // display output as it goes
		    //
		    BufferedReader br = new BufferedReader(new InputStreamReader(ct.getInputStream()));
		    String line;
		    while ((line = br.readLine ()) != null) {
			System.out.println("latex: " + line);
		    }

		    ct.waitFor();

		    System.out.println();
                    System.out.print(".");
                    rt = Runtime.getRuntime();  
		    cmd = "dvips -o " + tmp + File.separator + "latextmp.ps " + tmp+File.separator+"latextmp.dvi" ;

                    ct = rt.exec(cmd,null,new File(tmp));
		    ct.waitFor();
                    System.out.print(".");
                    rt = Runtime.getRuntime();
		    **/

		    /**
                    if (sun)
                    cmd = "perl /afs/athena.mit.edu/software/tth/arch/sun4x_56/bin/ps2gif " + tmp + File.separator + "latextmp.ps " +tmp + File.separator + "latextmp"+c+".gif";
                    else
                     cmd = "perl /afs/athena.mit.edu/software/tth/arch/i386_linux24/bin/ps2gif " + tmp + File.separator + "latextmp.ps " +tmp + File.separator + "latextmp"+c+".gif";
		    **/

		    //cmd = "gs -sDEVICE=png256 -dNOPAUSE -sOutputFile=" + tmp + File.separator + "latextmp" + c + ".png " + tmp + File.separator + "latextmp.ps -c quit";


		    //String input = tmp + File.separator + "latextmp.ps";
		    String input = postScriptLocation;
		    String output = tmp + File.separator + "latextmp" + c + ".gif";

		    String cmd = "convert.bat \"" + gsLocation + "\" \"" + input + "\" " + output;

		    System.out.println("About to run [" + cmd + "]");
		    
		    Runtime rt = Runtime.getRuntime();
                    Process ct = rt.exec(cmd);

		    String line = "";
		    BufferedReader br = new BufferedReader(new InputStreamReader(ct.getInputStream()));
		    while ((line = br.readLine ()) != null) {
			System.out.println(line);
		    }

		    ct.waitFor();

                    System.out.print(".");
                    BufferedImage bi = ImageIO.read(new File(tmp+File.separator+"latextmp"+c+".gif")); // was gif
                    PSImageData psi = null;
                    if (titleset)
			psi = new PSImageData(bi,0,0,0);
                    else if (hintset)
                        psi = new PSImageData(bi,probnum,qgnum,hintnum);
                    else
                        psi = new PSImageData(bi,probnum,qgnum,quest); 
                    
                    vs.add(psi);
                    
                } catch (Exception e) {e.printStackTrace();}
                break;
                
	}

            /*
            case Node.TEXT_NODE: {
		//  Text text = (Text)node;
                //System.out.println("Content: " + text.getData());        
                break;
            }
            */
        }

    } // count(Node)

}

class GifPanel extends JPanel {
    Vector giflist;
    public GifPanel(Vector gifv) {
        super();
        this.giflist = gifv;
        int ycount = 0;
        for (int i = 0;i <giflist.size();i++) {
	    PSImageData psi = ((PSImageData)giflist.get(i));
            BufferedImage a = psi.getImage();
            ycount+=2+a.getHeight();
        }
        setPreferredSize(new Dimension(500,ycount));
        setBackground(Color.white);
   }
    
    
 
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2 = (Graphics2D)g;
        int ycount = 0;
        for (int i = 0;i <giflist.size();i++) {
            PSImageData psi = ((PSImageData)giflist.get(i));
            BufferedImage a =  psi.getImage();
	    g2.drawImage(a,null,300,ycount);
            String tmp = "Problem: " + psi.getProbIdx();
            g2.drawString(tmp, 0, ycount+20);
            tmp = "QuestGroup: " + psi.getQGIdx();
            g2.drawString(tmp, 80, ycount+20);
            tmp = "Quest: " + psi.getQuestIdx();
            g2.drawString(tmp, 180, ycount+20);
       
            ycount+=2+a.getHeight();
        }
    }
}
